@@ -194,6 +194,10 @@ pub fn split_on_newlines(s: &str) -> Vec<&str> {
         .collect()
 }
 
+pub fn is_all_whitespace(s: &str) -> bool {
+    s.chars().all(|c| c.is_whitespace())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -323,4 +327,9 @@ mod tests {
     fn test_split_line_with_trailing_newline() {
         assert_eq!(split_on_newlines("foo\nbar\n"), vec!["foo", "bar", ""]);
     }
+
+    #[test]
+    fn test_is_all_whiteapce() {
+        assert!(is_all_whitespace(" \n\t"));
+    }
 }
