@@ -57,7 +57,7 @@ use parse::guess_language::{guess, language_name};
 static GLOBAL: MiMalloc = MiMalloc;
 
 use diff::sliders::fix_all_sliders;
-use options::{DiffOptions, DisplayMode, DisplayOptions, FileArgument, Mode, DEFAULT_TAB_WIDTH};
+use options::{DiffOptions, DisplayMode, DisplayOptions, FileArgument, Mode};
 use owo_colors::OwoColorize;
 use rayon::prelude::*;
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -100,8 +100,6 @@ fn main() {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
-            // TODO: Load display options rather than hard-coding.
-            let src = replace_tabs(&src, DEFAULT_TAB_WIDTH);
 
             let language = language_override.or_else(|| guess(path, &src));
             match language {
@@ -122,8 +120,6 @@ fn main() {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
-            // TODO: Load display options rather than hard-coding.
-            let src = replace_tabs(&src, DEFAULT_TAB_WIDTH);
 
             let language = language_override.or_else(|| guess(path, &src));
             match language {
@@ -258,16 +254,6 @@ fn main() {
     };
 }
 
-/// Return a copy of `str` with all the tab characters replaced by
-/// `tab_width` strings.
-///
-/// TODO: This break parsers that require tabs, such as Makefile
-/// parsing. We shouldn't do this transform until after parsing.
-fn replace_tabs(src: &str, tab_width: usize) -> String {
-    let tab_as_spaces = " ".repeat(tab_width);
-    src.replace('\t', &tab_as_spaces)
-}
-
 /// Print a diff between two files.
 fn diff_file(
     lhs_display_path: &str,
@@ -323,10 +309,6 @@ fn diff_file_content(
         (ProbableFileKind::Text(lhs_src), ProbableFileKind::Text(rhs_src)) => (lhs_src, rhs_src),
     };
 
-    // TODO: don't replace tab characters inside string literals.
-    lhs_src = replace_tabs(&lhs_src, display_options.tab_width);
-    rhs_src = replace_tabs(&rhs_src, display_options.tab_width);
-
     // Ignore the trailing newline, if present.
     // TODO: highlight if this has changes (#144).
     // TODO: factor out a string cleaning function.
@@ -343,7 +325,7 @@ fn diff_file_content(
         FileArgument::DevNull => (&lhs_src, Path::new(&lhs_display_path)),
     };
 
-    let language = language_override.or_else(|| guess(guess_path, guess_src));
+    let mut language = language_override.or_else(|| guess(guess_path, guess_src));
     let lang_config = language.map(tsp::from_language);
 
     if lhs_bytes == rhs_bytes {
@@ -432,6 +414,7 @@ fn diff_file_content(
             if exceeded_graph_limit {
                 let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
                 let rhs_positions = line_parser::change_positions(&rhs_src, &lhs_src);
+                language = None;
                 (
                     Some("Text (exceeded DFT_GRAPH_LIMIT)".into()),
                     lhs_positions,
