@@ -9,7 +9,7 @@ use crate::{
     diff::changes::ChangeKind,
     diff::changes::{ChangeKind::*, ChangeMap},
     diff::myers_diff,
-    lines::{LineNumber, NewlinePositions},
+    lines::{is_all_whitespace, LineNumber, NewlinePositions},
     positions::SingleLineSpan,
 };
 use Syntax::*;
@@ -637,7 +637,7 @@ pub fn split_words(s: &str) -> Vec<&str> {
     let mut res = vec![];
     let mut word_start = None;
     for (idx, c) in s.char_indices() {
-        if c.is_alphanumeric() {
+        if c.is_alphanumeric() || c == '-' || c == '_' {
             if word_start.is_none() {
                 word_start = Some(idx);
             }
@@ -678,16 +678,18 @@ fn split_comment_words(
         match diff_res {
             myers_diff::DiffResult::Left(word) => {
                 // This word is novel to this side.
-                res.push(MatchedPos {
-                    kind: MatchKind::NovelWord {
-                        highlight: TokenKind::Atom(AtomKind::Comment),
-                    },
-                    pos: content_newlines.from_offsets_relative_to(
-                        pos,
-                        offset,
-                        offset + word.len(),
-                    )[0],
-                });
+                if !is_all_whitespace(word) {
+                    res.push(MatchedPos {
+                        kind: MatchKind::NovelWord {
+                            highlight: TokenKind::Atom(AtomKind::Comment),
+                        },
+                        pos: content_newlines.from_offsets_relative_to(
+                            pos,
+                            offset,
+                            offset + word.len(),
+                        )[0],
+                    });
+                }
                 offset += word.len();
             }
             myers_diff::DiffResult::Both(word, opposite_word) => {
